/* This `webattery` is a battery power displaying program.
 * Copyright (C) 2009, 2010 Alagunambi Welkin <alagunambiwelkin@gmail.com>
 *  
 * This file is part of Webattery.
 *
 * Webattery is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Webattery is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Webattery.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "webattery.h"

int main (int argc, char **argv) 
{
  short int		next_option;
  short int		ultimate_option;
	
  ultimate_option = find_dirs();

  if (ultimate_option == 1)
  {
    fprintf(stderr,"\nNo Battery\n\n");
    exit (EXIT_FAILURE);
  }
    
  while (ultimate_option >= 3) 
  {
    dir = dir_list[ultimate_option - 1]->d_name;
    ultimate_option--;

    sprintf(file1_path,"/proc/acpi/battery/%s/state",dir);
    sprintf(file2_path,"/proc/acpi/battery/%s/info",dir);

    const char* const short_options = "hcv";

    const struct option long_options[] = 
      {
      { "help",    0, NULL, 'h' },
      { "verbose", 0, NULL, 'v' },
      { "version", 0, NULL, '1' },
      { "compact", 0, NULL, 'c' },
      {  NULL,     0, NULL,  0  }
      };

    program_name = argv[0];
  
    file_work(file1_path);

    string_match("remaining capacity:", 'a');
    string_match("charging state:", 'b');

    file_work(file2_path);

    string_match("last full capacity:", 'c');

    BATT = (we_batt.we_remaining_capacity*100)/
           we_batt.we_last_full_capacity;
  
    if (argc > 1) 
    {
      do 
      {
        next_option = getopt_long(
          argc,
          argv,
          short_options,
          long_options,
          NULL );
			
        switch (next_option) 
        {
          case 'h':
           print_usage(stdout,0);

          case 'v':
	   we_verbose();
	  break;
	
	  case 'c':
	   output_type('c');
	  break;

	  case '1':
	   fprintf(stdout,"Webattery  Copyright (C) 2009, 2010 "
	   "Alagunambi Welkin <alagunambiwelkin@gmail.com>\n\n"
           WEBATTERY"\n\n"
	   "You can get this project latest release "
	   "from\nhttp://webattery.sourceforge.net\n\n"
	   "This program comes with ABSOLUTELY NO WARRANTY.\n" 
	   "This is free software, and you are "
	   "welcome to redistribute it\n"
	   "under certain conditions. "
	   "For details see GNU General Public License.\n\n"
	   "You should have received a copy of "
	   "the GNU General Public License\n"
	   "along with Webattery. If not, "
	   "see <http://www.gnu.org/licenses/>\n\n"
	   "Report bug at <webttery-mailinglist@lists.sf.net>.\n");
	 
          break;

	  case '?':
	   print_usage(stderr,1);

	  case -1:
	  break;

	  default:
	   abort();
        }
      }
      while (next_option != -1);
    }
    else 
    {
      output_type('n');
    }
  }
  exit (EXIT_SUCCESS); 
}

/**********************************************************************
 This function finds the no. of directories beyond /proc/acpi/battery/? 
**********************************************************************/

int find_dirs (void) 
{
  int			total_dirs;
  
  total_dirs = scandir(PRE_FILE_PATH, &dir_list, 0, alphasort);
	
  if (total_dirs < 3) 
  {
    return (EXIT_FAILURE);
  }
  else 
  {
    return (total_dirs);
  }
}

/**********************************************************************
 This function displays the help message. 
**********************************************************************/

void print_usage (FILE *stream, int exit_code) 
{
  fprintf (stream, 
    "\nUsage:  %s [ options... ]\n", 
    program_name);

  fprintf (stream,
    " -h --help\t Display this usage information.\n"
    " -v --verbose\t Verbose mode.\n"
    " -c --compact\t Display in compact mode.\n"
    "    --version\t Display Version.\n\n"
    "Report bugs to <webattery-mailinglist@lists.sf.net>.\n");

  exit(exit_code);
}

/**********************************************************************
 This function will open and close the required files.
**********************************************************************/

int file_work (char *file_path) 
{
  FILE 			*file;

  file = fopen(file_path,"r");
  file_content = fread(temp,1,sizeof(temp),file);
  fclose (file);

  if (file_content == 0 || file_content == sizeof(temp)) 
  {
    fprintf(stderr,"Empty file\n");
    return (EXIT_FAILURE);
  }
  else
  {
    return (EXIT_SUCCESS);
  }
}

/**********************************************************************
 This function extracts the required string from the opened file.
**********************************************************************/

void string_match (char *match_string, char choice) 
{ 
  temp[file_content] = '\0';
  match = strstr (temp, match_string);

  if (match == NULL) 
  {
    fprintf(stderr,"Match not found\n");
    exit (EXIT_FAILURE);
  }
  else 
  {
    switch (choice)  
    {
      case 'a':
       sscanf(match, "remaining capacity: %d %s",
              &we_batt.we_remaining_capacity,
              we_unit.we_unit_remaining_capacity);
      break;
			
      case 'b':
       sscanf(match, "charging state: %s",we_batt.we_charging_state);
      break;
			
      case 'c':
       sscanf(match,"last full capacity: %d %s",
              &we_batt.we_last_full_capacity,
              we_unit.we_unit_last_full_capacity);
      break;
        
      case 'd':
       sscanf(match,"design capacity: %s %s",
              we_batt.we_design_capacity,
              we_unit.we_unit_design_capacity);
      break;

      case 'e':
       sscanf(match,"battery technology: %s",
              we_batt.we_battery_technology);
      break;

      case 'f':
       sscanf(match,"design voltage: %s %s",
              we_batt.we_design_voltage,
              we_unit.we_unit_design_voltage);
      break;

      case 'g':
       sscanf(match,"design capacity warning: %s %s",
              we_batt.we_design_capacity_warning,
              we_unit.we_unit_design_capacity_warning);
      break;

      case 'h':
       sscanf(match,"design capacity low: %s %s",
              we_batt.we_design_capacity_low,
              we_unit.we_unit_design_capacity_low);
      break;

      case 'i':
       sscanf(match,"model number: %s",we_batt.we_model_number);
      break;

      case 'j':
       sscanf(match,"serial number: %s %s",
              we_batt.we_serial_number, 
              we_batt.we_manufactured_date);
      break;

      case 'k':
       sscanf(match,"battery type: %s",we_batt.we_battery_type);
      break;

      case 'l':
       sscanf(match,"OEM info: %s",we_batt.we_oem_info);
      break;

      case 'm':
       sscanf(match,"capacity state: %s",we_batt.we_capacity_state);
      break;

      case 'n':
       sscanf(match,"present rate: %d %s",
              &we_batt.we_present_rate,
              we_unit.we_unit_present_rate);
      break;

      case 'o':
       sscanf(match,"present voltage: %s %s",
              we_batt.we_present_voltage,
              we_unit.we_unit_present_voltage);
      break;
    }
  } 
}

/**********************************************************************
 This function is for getting the total output string length.
**********************************************************************/

int total_str_len (void) 
{ 
  sprintf(display_out,"  %d% & %s  ", BATT, we_batt.we_charging_state);
  total_string_len = strlen(display_out);
  
  if (total_string_len <= 0)
  {
    return (EXIT_FAILURE);
  }
  else
  {
    return (EXIT_SUCCESS);
  }
}

/**********************************************************************
 This function is for printing the border of output.
**********************************************************************/

int border (void) 
{
  unsigned short int	check1;
  
  check1 = total_str_len();
  if (check1 == 0)
  {
    for (i = 0;i <= total_string_len;i++) 
    {
      fprintf(stdout,"\033[%d;7m \033[0m",BC);
    }

    fprintf(stdout,"\033[%d;7m \033[0m\n",BC);
    return (EXIT_SUCCESS);
  }
  else
  {
    return (EXIT_FAILURE);
  }
}

/**********************************************************************
 This function is for printint middle output.
**********************************************************************/

int batt_output (int colour_val, int some) 
{
  unsigned short int	tmp_1;
  unsigned short int	tmp_2;
  unsigned short int	check2;
  
  check2 = total_str_len();
  
  if (check2 == 0)
  {
    tmp_2 = colour_val;

    for (i = 0;i <= total_string_len;i++) 
    {
      tmp_1 = i*100/total_string_len;
	
      if (tmp_1 >= BATT) 
      {
        colour_val = 39;
      }
  
      fprintf(stdout,"\033[%d;7m%c\033[0m",colour_val,display_out[i]);
    }
  
    if (some != 1)
    {
      fprintf(stdout,"\033[%d;7m  \033[0m",BC);
    }
  
    fprintf(stdout,"\033[%d;7m \033[0m",tmp_2);

    if (some != 0)
    {
      fprintf(stdout,"\n");
    }
    return (EXIT_SUCCESS);
  }
  else
  {
    exit (EXIT_FAILURE);
  }
}

/**********************************************************************
 This function is for selecting colour of progress bar.
**********************************************************************/

void colour_print (char chuma)
{
  unsigned short int	x;

  if (chuma == 'v')
  {
    x = 0;
  }
  else if (chuma == 'c')
  {
    x = 1;
  }
  else
  {
    x = 2;
  }

  if (BATT != '\0') 
  {
    if (BATT == 100) 
    {
      batt_output(32,x);
    }
    else if (BATT < 40) 
    {
      batt_output(31,x);
    }
    else 
    {
      batt_output(33,x);
    }
  } 
  else 
  {
    batt_output(35,x);
  }
}

/**********************************************************************
 This function is for choosing type of output.
**********************************************************************/

void output_type (char type) 
{
  switch (type) 
  {
    case 'c':
     colour_print('c');
    break;
    
    case 'n':
     border();
     colour_print('n');
     border();
    break;
    
    case 'v':
     border();
     colour_print('v');
     fprintf(stdout,"---> %s\n", we_batt.we_charging_state);
     border();
    break;
  }
}

/**********************************************************************
 This function is for displaying the verbose output.
**********************************************************************/

int we_verbose(void) 
{
  unsigned int		check3;
  unsigned int		check4;

  check3 = file_work(file2_path);
  
  if (check3 == 0)
  {
    string_match("design capacity:",'d');
    string_match("battery technology:",'e');
    string_match("design voltage:",'f');
    string_match("design capacity warning:",'g');
    string_match("design capacity low:",'h');
    string_match("model number:",'i');
    string_match("serial number:",'j');
    string_match("battery type:",'k');
    string_match("OEM info:",'l');
    
    check4 = file_work(file1_path);
  }
  else
  { 
    exit (EXIT_FAILURE);
  }
 
  if (check4 == 0)
  {
    string_match("capacity state:",'m');
    string_match("present rate:",'n');
    string_match("present voltage:",'o');
  }  
  else
  {
    exit (EXIT_FAILURE);
  }
  
  fprintf(stdout,"Battery Informations:\n");
  fprintf(stdout,"Estimated Time for %s:\t%d minute(s)\n",
	 we_batt.we_charging_state,
         batt_time());
  fprintf(stdout,"OEM:\t\t\t\t%s\n",we_batt.we_oem_info);
  fprintf(stdout,"Type:\t\t\t\t%s\n",we_batt.we_battery_type);
  fprintf(stdout,"Technology:\t\t\t%s\n",
	 we_batt.we_battery_technology);
  fprintf(stdout,"Serial Number:\t\t\t%s\n",we_batt.we_serial_number);
  fprintf(stdout,"Manufactured Date:\t\t%s\n",
	 we_batt.we_manufactured_date);
  fprintf(stdout,"Design Capacity:\t\t%s %s\n",
         we_batt.we_design_capacity,
         we_unit.we_unit_design_capacity);
  fprintf(stdout,"Design Voltage:\t\t\t%s %s\n",
         we_batt.we_design_voltage,
         we_unit.we_unit_design_voltage);
  fprintf(stdout,"Design Low Capacity:\t\t%s %s\n",
         we_batt.we_design_capacity_low,
         we_unit.we_unit_design_capacity_low);
  fprintf(stdout,"Design Warning Capacity:\t%s %s\n",
         we_batt.we_design_capacity_warning,
         we_unit.we_unit_design_capacity_warning);
  fprintf(stdout,"     ,--> %d %s\n""    /\n",
         we_batt.we_remaining_capacity,
         we_unit.we_unit_remaining_capacity);

  output_type('v');

  fprintf(stdout,"\t\t  /\n""     %d %s <--*\n",
         we_batt.we_last_full_capacity,
         we_unit.we_unit_last_full_capacity);
  fprintf(stdout,"Model Number:\t\t\t%s\n",we_batt.we_model_number);
  fprintf(stdout,"Capacity State:\t\t\t%s\n",
	 we_batt.we_capacity_state);
  fprintf(stdout,"Charging State:\t\t\t%s\n",
	 we_batt.we_charging_state);
  fprintf(stdout,"Present Rate:\t\t\t%d %s\n",
         we_batt.we_present_rate,
         we_unit.we_unit_present_rate);
  fprintf(stdout,"Present Voltage:\t\t%s %s\n",
         we_batt.we_present_voltage,
         we_unit.we_unit_present_voltage);
  fprintf(stdout,"Last Full Capacity:\t\t%d %s\n",
         we_batt.we_last_full_capacity,
         we_unit.we_unit_last_full_capacity);
  fprintf(stdout,"Remaining Capacity:\t\t%d %s\n",
         we_batt.we_remaining_capacity,
         we_unit.we_unit_remaining_capacity);

  exit (EXIT_SUCCESS);
}

/**********************************************************************
 This function is for estimating charging and discharging time.
**********************************************************************/

int batt_time(void)
{
  unsigned short int	charging;
  unsigned short int	discharging;
  unsigned short int	charged;

  if (strncmp(we_batt.we_charging_state, "chargi", 6) == 0)
  {
    charging = 1;
    discharging = 0;
    charged = 0;
  }
  else if (strncmp(we_batt.we_charging_state, "d", 1) == 0)
  {
    charging = 0;
    discharging = 1;
    charged = 0;
  }
  else
  {  
    charging = 0;
    discharging = 0;
    charged = 1;
  }

  return (60 * ((((float)(we_batt.we_last_full_capacity -
	  we_batt.we_remaining_capacity) * charging) /
          (we_batt.we_present_rate + charged)) +
	  (((float)we_batt.we_remaining_capacity /
	  (float)we_batt.we_last_full_capacity) * discharging)));
}
